package org.kie.examples.osgi;

import org.kie.api.KieBase;
import org.kie.api.KieBaseConfiguration;
import org.kie.api.KieServices;
import org.kie.api.builder.KieBuilder;
import org.kie.api.builder.KieFileSystem;
import org.kie.api.builder.ReleaseId;
import org.kie.api.builder.model.KieModuleModel;
import org.kie.api.io.Resource;
import org.kie.api.runtime.KieSession;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;

/**
 *
 * Sample OSGi BundleActivator initializing Drools KieBase with custom classloader
 * (BZ ).
 */
public class SampleKieOsgiActivator implements BundleActivator {

    private static final ReleaseId SAMPLE_APP_REL_ID = KieServices.Factory.get()
            .newReleaseId("org.kie.examples", "kie-osgi-sample", "1.0.0-SNAPSHOT");

    private KieSession ksession;

    public void start(final BundleContext bc) throws Exception {
        KieServices ks = KieServices.Factory.get();
        KieBaseConfiguration kbaseConfig = ks.newKieBaseConfiguration(null, this.getClass().getClassLoader());
        // the following does not work:
        KieBase kbase = ks.newKieClasspathContainer().newKieBase(kbaseConfig);
        // this workaround works:
        //KieBase kbase = this.createKieBase(kbaseConfig);

        this.ksession = kbase.newKieSession();

        System.out.println("KieSession created.");
    }

    // basically a workaround for problem with ks.newKieClasspathContainer().newKieBase(kbaseConfig);
    private KieBase createKieBase(final KieBaseConfiguration config) {
        final String drl = "package org.kie.examples.osgi\n"
                + "rule testRule \n"
                + "when String()\n"
                + "then end\n";

        KieServices ks = KieServices.Factory.get();
        Resource drlResource = ks.getResources().newByteArrayResource(drl.getBytes());
        drlResource.setSourcePath("src/main/resources/org/kie/examples/osgi/sample.drl");

        KieModuleModel kmodule = ks.newKieModuleModel();
        kmodule.newKieBaseModel("defaultBase")
                .addPackage("org.kie.examples.osgi")
                .setDefault(true)
                .newKieSessionModel("defaultSession")
                    .setDefault(true);

        KieFileSystem kfs = ks.newKieFileSystem();
        kfs.generateAndWritePomXML(SAMPLE_APP_REL_ID);
        kfs.write(drlResource);
        kfs.writeKModuleXML(kmodule.toXML());

        KieBuilder kbuilder = ks.newKieBuilder(kfs);
        kbuilder.buildAll();

        return ks.newKieContainer(SAMPLE_APP_REL_ID).getKieBase();
    }

    public void stop(final BundleContext bc) throws Exception {
        if (this.ksession != null) {
            this.ksession.dispose();
            System.out.println("KieSession disposed.");
        }
    }

}
